/**
 * @file
 * VuoList_VuoFileType C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoFileType_H
#define VuoList_VuoFileType_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoFileType VuoList_VuoFileType
 * A list of @ref VuoFileType elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoFileType_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoFileType elements.
 */
typedef const struct VuoList_VuoFileType_struct { void *l; } * VuoList_VuoFileType;
/// @}
#endif

/**
 * Creates a new list of @ref VuoFileType elements.
 */
VuoList_VuoFileType VuoListCreate_VuoFileType(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoFileType VuoListCopy_VuoFileType(const VuoList_VuoFileType list);

/**
 * Returns the @ref VuoFileType at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoFileType VuoListGetValue_VuoFileType(const VuoList_VuoFileType list, const unsigned long index);

/**
 * Changes the @ref VuoFileType at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoFileType(const VuoList_VuoFileType list, const VuoFileType value, const unsigned long index);

/**
 * Inserts the `VuoFileType` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoFileType(const VuoList_VuoFileType list, const VuoFileType value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoFileType(VuoList_VuoFileType list, const VuoFileType value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoFileType(VuoList_VuoFileType list, const VuoFileType value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoFileType(VuoList_VuoFileType list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoFileType(VuoList_VuoFileType list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoFileType(VuoList_VuoFileType list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoFileType(VuoList_VuoFileType list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoFileType(VuoList_VuoFileType list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoFileType(VuoList_VuoFileType list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoFileType(VuoList_VuoFileType list);

/**
 * Removes the `VuoFileType` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoFileType(VuoList_VuoFileType list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoFileType(const VuoList_VuoFileType list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoFileType VuoList_VuoFileType_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoFileType_getJson(const VuoList_VuoFileType value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoFileType_getSummary(const VuoList_VuoFileType value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoFileType VuoList_VuoFileType_makeFromString(const char *str);
char * VuoList_VuoFileType_getString(const VuoList_VuoFileType value);
/// @}

/**
 * @}
 */

#endif
